import amqp, { ConsumeMessage } from "amqplib";
import { ITodo } from "../interfaces/ITodo";
import { MongoDbModel } from "../mongodb/MongoDb";
import { EnvService } from "../services/EnvService";

export class RabbitMQ {
  channel: amqp.Channel;
  queueName: string;
  exchangeName: string;

  mongoClient: MongoDbModel;
  envService: EnvService;

  constructor() {
    this.envService = EnvService.getInstance();
    this.queueName = this.envService.getEnvVariable("RABBITMQ_QUEUE_NAME");

    this.mongoClient = new MongoDbModel();

    this.connect()
      .then(() => {
        console.log("RabbitMQ connected");
      })
      .catch((error) => {
        console.error("Error connecting to RabbitMQ:", error);
      });
  }

  async connect() {
    try {
      const connection = await amqp.connect({
        protocol: "amqp",
        hostname: this.envService.getEnvVariable("RABBITMQ_HOST"),
        port: parseInt(this.envService.getEnvVariable("RABBITMQ_PORT")),
        username: this.envService.getEnvVariable("RABBITMQ_USERNAME"),
        password: this.envService.getEnvVariable("RABBITMQ_PASSWORD"),
      });

      this.channel = await connection.createChannel();
      await this.channel.assertQueue(this.queueName);
    } catch (error) {
      console.error("Error connecting to RabbitMQ:", error);
      throw error;
    }
  }

  async startConsumer(callback: Function): Promise<ITodo | void> {
    this.channel.assertQueue(this.queueName);
    this.channel.consume(this.queueName, (message: ConsumeMessage | null) => {
      if (message) {
        const todo: ITodo = JSON.parse(message.content.toString()).payload;
        this.channel.ack(message);
        callback(todo);
      }
    });
  }
}
