import express from "express";
import mongoose from "mongoose";
import todoRouter from "./routes/todoRouter";
import { EnvService } from "./services/EnvService";
import { ApiError } from "./utils/ApiError";

class TodoApp {
  app: express.Application;
  envService: EnvService;

  constructor() {
    this.envService = EnvService.getInstance();
    this.connectToDB();
  }

  public run() {
    this.app = express();
    this.setMiddlewares();
    this.setRoutes();
    this.startServer();
  }

  private setRoutes() {
    this.app.use("/todo", todoRouter);
    this.app.all("*", (req, res, next) => {
      const error = new ApiError("Are you lost?", 404, "Not Found");
      next(error);
    });
  }

  private startServer() {
    const PORT = this.envService.getEnvVariable("PORT") || 3000;
    this.app.listen(PORT, () => {
      console.log(`Server started on port ${PORT}`);
    });
  }

  private setMiddlewares() {
    this.app.use(express.json());
  }

  private connectToDB() {
    mongoose.connect(this.envService.getEnvVariable("DATABASE_URL"));
    const db = mongoose.connection;
    // Check for DB connection
    db.on("error", () => {
      console.error.bind(console, "MongoDB connection error:");
      process.exit(1);
    });
    db.once("open", () => {
      console.log("Connected to MongoDB");
    });
  }
}

const app = new TodoApp();
app.run();
