import { Request, Response, NextFunction } from "express";
import { ApiError } from "../utils/ApiError";
import { DateTime } from "luxon";
import mongoose from "mongoose";
import { DateService } from "../services/DateService";

const currentDate = DateService.getInstance().getCurrentDate();

const createTodoMiddleWare = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { title, description, due_date } = req.body;
  if (!title || !due_date) {
    const error = new ApiError(
      `${!title ? "title" : "due_date"} is required`,
      400,
      "Bad Request"
    );
    return next(error);
  }
  //check if date is valid, this is valid: 2023-07-08T14:00:00.000Z
  const date = DateTime.fromISO(due_date);
  if (!date.isValid) {
    const error = new ApiError(
      `due_date must be ISO Format`,
      400,
      "Bad Request"
    );
    return next(error);
  }
  if (new Date(due_date) < currentDate) {
    const error = new ApiError(
      `due_date must be greater than current date`,
      400,
      "Bad Request"
    );
    return next(error);
  }

  if (!description) {
    req.body.description = "";
  }

  next();
};

const paramIdMiddleware = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { id } = req.params;
  if (!mongoose.Types.ObjectId.isValid(id)) {
    const error = new ApiError(
      `id must be a valid mongo id`,
      400,
      "Bad Request"
    );
    return next(error);
  }
  next();
};

export { createTodoMiddleWare, paramIdMiddleware };
