import { Todo, ITodo } from "../schemas/todoSchema";
import { ApiError } from "../utils/ApiError";

interface Todo {
  title: string;
  description: string;
  due_date: Date;
}

export class TodoModel {
  public findAll = async (): Promise<ITodo[]> => {
    const todos = await Todo.find();
    return todos;
  };

  public findOne = async (_id: string): Promise<ITodo | ApiError> => {
    try {
      const todo = await Todo.findById(_id);
      if (!todo) {
        const error = new ApiError("Todo not found", 404, "Not Found");
        return error;
      }
      return todo;
    } catch {
      const error = new ApiError(
        "Internal server error",
        500,
        "Internal Server Error"
      );
      return error;
    }
  };

  public create = async (params: Todo): Promise<ITodo | ApiError> => {
    let todo = new Todo(params);
    todo = await todo.save();
    if (!todo) {
      const error = new ApiError(
        "Internal server error",
        500,
        "Internal Server Error"
      );
      return error;
    }
    return todo;
  };

  public update = async (
    params: Todo,
    _id: string
  ): Promise<ITodo | ApiError> => {
    const todo = await Todo.findOne({ _id });
    if (!todo) {
      const error = new ApiError("Todo not found", 404, "Not Found");
      return error;
    }

    todo.title = params.title;
    todo.description = params.description;
    todo.due_date = params.due_date;

    await todo.save();
    return todo;
  };

  public remove = async (_id: string): Promise<boolean | ApiError> => {
    const result = await Todo.deleteOne({ _id });
    return result.deletedCount > 0
      ? true
      : new ApiError("Todo not found", 404, "Not Found");
  };

  public removeAll = async (): Promise<boolean> => {
    await Todo.deleteMany({});
    return true;
  };
}
